searchNodes=[{"ref":"Rambo.html","title":"Rambo","type":"module","doc":"Ever stumped trying to get output from a port? Rambo has one mission. Start your program, pipe standard input, send EOF and return with output. Rambo.run(&quot;cat&quot;, in: &quot;hello&quot;) {:ok, %Rambo{out: &quot;hello&quot;}} Rambo.run(&quot;echo&quot;, [&quot;-n&quot;, &quot;world&quot;]) {:ok, %Rambo{out: &quot;world&quot;}} Rambo.run(&quot;ls&quot;) |&gt; Rambo.run(&quot;sort&quot;) |&gt; Rambo.run(&quot;head&quot;) {:ok, %Rambo{out: &quot;bar\\nbaz\\nfoo\\n&quot;}} One mission, one function. Why? Erlang ports do not work with programs that expect EOF to produce output. The only way to close standard input is to close the port, which also closes standard output preventing results from coming back to your app. This gotcha is marked Won’t Fix. Design When Rambo is asked to run a command, it creates a port to a shim. Then the shim runs your command, closes standard input and waits for output. After your command exits, its output is returned to your app before the port is closed and the shim exits. +-----------------+ stdin | +------+------+ --&gt; +---------+ | Erlang | Port | Shim | | Command | | +------+------+ &lt;-- +---------+ +-----------------+ stdout If the Erlang node stops during the command, the shim exits to avoid becoming an orphan (process leak) but not before the command finishes. Rambo does not start a pool of processes nor support bidirectional communication with your commands. It is intentionally kept simple and lightweight to run transient jobs with minimal overhead, such as calling a Python or Node script to transform some data. For more complicated use cases, see other libraries below. Caveats You cannot call Rambo.run from a GenServer because Rambo uses receive, which interferes with GenServer’s receive loop. However, you can wrap the call in a Task. task = Task.async(fn -&gt; Rambo.run(&quot;thingamabob&quot;) end) Task.await(task) Comparisons System.cmd If you don’t need to pipe standard input to your external program, just use System.cmd. Porcelain Porcelain cannot send EOF to trigger output by default. The Goon driver must be installed separately to add this capability. Rambo ships with the required native binaries. Goon is written in Go, a multithreaded runtime with a garbage collector. To be as lightweight as possible, Rambo’s shim is written in Rust. Single threaded, no garbage collector, no runtime overhead. Most importantly, Goon currently leaks processes. This isn’t an intractable problem, in fact writing a new driver to replace Goon should fix it. But Porcelain appears to be abandoned and I need this in production so effort went into creating Rambo. erlexec erlexec is great if you want fine grain control over external programs. Each external OS process is mirrored as an Erlang process, so you get asynchronous and bidirectional communication. You can kill your OS processes with any signal or monitor them for termination, among many powerful features. Choose erlexec if you want a kitchen sink solution. Installation Add rambo to your list of dependencies in mix.exs: def deps do [ {:rambo, &quot;~&gt; 0.1&quot;} ] end This package bundles macOS, Linux and Windows binaries (x86-64 architecture only). For other environments, install the Rust compiler and add the :rambo compiler to mix.exs. def project do [ compilers: [:rambo] ++ Mix.compilers() ] end Ideally the Rust compiler should never be required, but cross-compiling Rust is still a work in progress. Links Documentation Hex License Rambo is released under MIT license."},{"ref":"Rambo.html#run/1","title":"Rambo.run/1","type":"function","doc":"Runs command. Executes the command and returns {:ok, %Rambo{}} or {:error, reason}. reason is a string if the child process failed to start, or a %Rambo{} struct if the child process started successfully but exited with a non-zero status. Multiple calls can be chained together with the |&gt; pipe operator to simulate Unix pipes. Rambo.run(&quot;ls&quot;) |&gt; Rambo.run(&quot;sort&quot;) |&gt; Rambo.run(&quot;head&quot;) If any command did not exit with 0, the rest will not be executed and the last executed result is returned in an :error tuple. See run/2 or run/3 to pass arguments or options. Examples iex&gt; Rambo.run(&quot;echo&quot;) {:ok, %Rambo{status: 0, out: &quot;\\n&quot;, err: &quot;&quot;}}"},{"ref":"Rambo.html#run/2","title":"Rambo.run/2","type":"function","doc":"Runs command with arguments or options. Arguments can be a string or list of strings. See run/3 for options. Examples iex&gt; Rambo.run(&quot;echo&quot;, &quot;john&quot;) {:ok, %Rambo{out: &quot;john\\n&quot;}} iex&gt; Rambo.run(&quot;echo&quot;, [&quot;-n&quot;, &quot;john&quot;]) {:ok, %Rambo{out: &quot;john&quot;}} iex&gt; Rambo.run(&quot;cat&quot;, in: &quot;john&quot;) {:ok, %Rambo{out: &quot;john&quot;}}"},{"ref":"Rambo.html#run/3","title":"Rambo.run/3","type":"function","doc":"Runs command with arguments and options. Options :in - pipe as standard input :cd - the directory to run the command in :env - map or list of tuples containing environment key-value as strings Examples iex&gt; Rambo.run(&quot;/bin/sh&quot;, [&quot;-c&quot;, &quot;echo $JOHN&quot;], env: %{&quot;JOHN&quot; =&gt; &quot;rambo&quot;}) {:ok, %Rambo{out: &quot;rambo\\n&quot;}}"},{"ref":"Rambo.html#t:args/0","title":"Rambo.args/0","type":"type","doc":""},{"ref":"Rambo.html#t:result/0","title":"Rambo.result/0","type":"type","doc":""},{"ref":"Rambo.html#t:t/0","title":"Rambo.t/0","type":"type","doc":""}]